@@ -8,9 +8,9 @@
 #define nsTextFrame_h__
 
 #include "mozilla/Attributes.h"
-#include "mozilla/gfx/2D.h"
 #include "mozilla/UniquePtr.h"
 #include "mozilla/dom/Text.h"
+#include "mozilla/gfx/2D.h"
 
 #include "nsIFrame.h"
 #include "nsISelectionController.h"
@@ -309,6 +309,14 @@ class nsTextFrame : public nsIFrame {
     return &mContent->AsText()->TextFragment();
   }
 
+  /**
+   * Check that the text in this frame is entirely whitespace. Importantly,
+   * this function considers non-breaking spaces (0xa0) to be whitespace,
+   * whereas nsTextFrame::IsEmpty does not. It also considers both one and
+   * two-byte chars.
+   */
+  bool IsEntirelyWhitespace() const;
+
   ContentOffsets CalcContentOffsetsFromFramePoint(const nsPoint& aPoint) final;
   ContentOffsets GetCharacterOffsetAtFramePoint(const nsPoint& aPoint);
 
@@ -334,9 +342,8 @@ class nsTextFrame : public nsIFrame {
                                    PeekWordState* aState,
                                    bool aTrimSpaces) final;
 
-  nsresult CheckVisibility(nsPresContext* aContext, int32_t aStartIndex,
-                           int32_t aEndIndex, bool aRecurse, bool* aFinished,
-                           bool* _retval) final;
+  // Helper method that editor code uses to test for visibility.
+  [[nodiscard]] bool HasVisibleText();
 
   // Flags for aSetLengthFlags
   enum { ALLOW_FRAME_CREATION_AND_DESTRUCTION = 0x01 };
@@ -359,7 +366,9 @@ class nsTextFrame : public nsIFrame {
 
   bool IsEmpty() final;
   bool IsSelfEmpty() final { return IsEmpty(); }
-  nscoord GetLogicalBaseline(mozilla::WritingMode aWritingMode) const final;
+  Maybe<nscoord> GetNaturalBaselineBOffset(
+      mozilla::WritingMode aWM, BaselineSharingGroup aBaselineGroup,
+      BaselineExportContext) const override;
 
   bool HasSignificantTerminalNewline() const final;
 
@@ -551,7 +560,7 @@ class nsTextFrame : public nsIFrame {
     virtual void NotifySelectionDecorationLinePathEmitted() {}
   };
 
-  struct PaintTextParams {
+  struct MOZ_STACK_CLASS PaintTextParams {
     gfxContext* context;
     Point framePt;
     LayoutDeviceRect dirtyRect;
@@ -578,6 +587,12 @@ class nsTextFrame : public nsIFrame {
   struct PaintShadowParams;
   struct PaintDecorationLineParams;
 
+  struct SelectionRange {
+    const SelectionDetails* mDetails;
+    gfxTextRun::Range mRange;
+    uint32_t mPriority;
+  };
+
   // Primary frame paint method called from nsDisplayText.  Can also be used
   // to generate paths rather than paint the frame's text by passing a callback
   // object.  The private DrawText() is what applies the text to a graphics
@@ -605,6 +620,12 @@ class nsTextFrame : public nsIFrame {
       const mozilla::UniquePtr<SelectionDetails>& aDetails,
       SelectionType aSelectionType);
 
+  SelectionTypeMask ResolveSelections(const PaintTextSelectionParams& aParams,
+                                      const SelectionDetails* aDetails,
+                                      nsTArray<SelectionRange>& aResult,
+                                      SelectionType aSelectionType,
+                                      bool* aAnyBackgrounds = nullptr) const;
+
   void DrawEmphasisMarks(gfxContext* aContext, mozilla::WritingMode aWM,
                          const mozilla::gfx::Point& aTextBaselinePt,
                          const mozilla::gfx::Point& aFramePt, Range aRange,
@@ -716,6 +737,8 @@ class nsTextFrame : public nsIFrame {
                   DrawTarget* aDrawTarget, ReflowOutput& aMetrics,
                   nsReflowStatus& aStatus);
 
+  nscoord ComputeLineHeight() const;
+
   bool IsFloatingFirstLetterChild() const;
 
   bool IsInitialLetterChild() const;
@@ -738,8 +761,7 @@ class nsTextFrame : public nsIFrame {
    */
   void NotifyNativeAnonymousTextnodeChange(uint32_t aOldLength);
 
-  void SetInflatedFontMetrics(nsFontMetrics*);
-  nsFontMetrics* InflatedFontMetrics() const { return mFontMetrics; }
+  nsFontMetrics* InflatedFontMetrics() const;
 
   nsRect WebRenderBounds();
 
@@ -749,13 +771,21 @@ class nsTextFrame : public nsIFrame {
   // (To be used only on the first textframe in the chain.)
   nsTextFrame* FindContinuationForOffset(int32_t aOffset);
 
+  void SetHangableISize(nscoord aISize);
+  nscoord GetHangableISize() const;
+  void ClearHangableISize();
+
+  void SetTrimmableWS(gfxTextRun::TrimmableWS aTrimmableWS);
+  gfxTextRun::TrimmableWS GetTrimmableWS() const;
+  void ClearTrimmableWS();
+
  protected:
   virtual ~nsTextFrame();
 
   friend class mozilla::nsDisplayTextGeometry;
   friend class mozilla::nsDisplayText;
 
-  RefPtr<nsFontMetrics> mFontMetrics;
+  mutable RefPtr<nsFontMetrics> mFontMetrics;
   RefPtr<gfxTextRun> mTextRun;
   nsTextFrame* mNextContinuation;
   // The key invariant here is that mContentOffset never decreases along
@@ -783,8 +813,20 @@ class nsTextFrame : public nsIFrame {
   };
   mutable SelectionState mIsSelected;
 
-  // Whether a cached continuations array is present.
-  bool mHasContinuationsProperty = false;
+  // Flags used to track whether certain properties are present.
+  // (Public to keep MOZ_MAKE_ENUM_CLASS_BITWISE_OPERATORS happy.)
+ public:
+  enum class PropertyFlags : uint8_t {
+    // Whether a cached continuations array is present.
+    Continuations = 1 << 0,
+    // Whether a HangableWhitespace property is present.
+    HangableWS = 1 << 1,
+    // Whether a TrimmableWhitespace property is present.
+    TrimmableWS = 2 << 1,
+  };
+
+ protected:
+  PropertyFlags mPropertyFlags = PropertyFlags(0);
 
   /**
    * Return true if the frame is part of a Selection.
@@ -828,7 +870,7 @@ class nsTextFrame : public nsIFrame {
     // thickness of the decoration line
     const mozilla::StyleTextDecorationLength mTextDecorationThickness;
     nscolor mColor;
-    uint8_t mStyle;
+    mozilla::StyleTextDecorationStyle mStyle;
 
     // The text-underline-position property; affects the underline offset only
     // if mTextUnderlineOffset is auto.
@@ -838,7 +880,8 @@ class nsTextFrame : public nsIFrame {
                    mozilla::StyleTextUnderlinePosition aUnderlinePosition,
                    const mozilla::LengthPercentageOrAuto& aUnderlineOffset,
                    const mozilla::StyleTextDecorationLength& aDecThickness,
-                   const nscolor aColor, const uint8_t aStyle)
+                   const nscolor aColor,
+                   const mozilla::StyleTextDecorationStyle aStyle)
         : mFrame(aFrame),
           mBaselineOffset(aOff),
           mTextUnderlineOffset(aUnderlineOffset),
@@ -902,6 +945,15 @@ class nsTextFrame : public nsIFrame {
   bool CombineSelectionUnderlineRect(nsPresContext* aPresContext,
                                      nsRect& aRect);
 
+  // This sets *aShadows to the appropriate shadows, if any, for the given
+  // type of selection.
+  // If text-shadow was not specified, *aShadows is left untouched.
+  // Note that the returned shadow(s) will only be valid as long as the
+  // textPaintStyle remains in scope.
+  void GetSelectionTextShadow(
+      SelectionType aSelectionType, nsTextPaintStyle& aTextPaintStyle,
+      mozilla::Span<const mozilla::StyleSimpleShadow>* aShadows);
+
   /**
    * Utility methods to paint selection.
    */
@@ -932,6 +984,7 @@ class nsTextFrame : public nsIFrame {
    * @return            true if the selection affects colors, false otherwise
    */
   static bool GetSelectionTextColors(SelectionType aSelectionType,
+                                     const nsAtom* aHighlightName,
                                      nsTextPaintStyle& aTextPaintStyle,
                                      const TextRangeStyle& aRangeStyle,
                                      nscolor* aForeground,
@@ -959,13 +1012,7 @@ class nsTextFrame : public nsIFrame {
 
   // Clear any cached continuations array; this should be called whenever the
   // chain is modified.
-  void ClearCachedContinuations() {
-    MOZ_ASSERT(NS_IsMainThread());
-    if (mHasContinuationsProperty) {
-      RemoveProperty(ContinuationsProperty());
-      mHasContinuationsProperty = false;
-    }
-  }
+  inline void ClearCachedContinuations();
 
   /**
    * UpdateIteratorFromOffset() updates the iterator from a given offset.
@@ -981,5 +1028,14 @@ class nsTextFrame : public nsIFrame {
 };
 
 MOZ_MAKE_ENUM_CLASS_BITWISE_OPERATORS(nsTextFrame::TrimmedOffsetFlags)
+MOZ_MAKE_ENUM_CLASS_BITWISE_OPERATORS(nsTextFrame::PropertyFlags)
+
+inline void nsTextFrame::ClearCachedContinuations() {
+  MOZ_ASSERT(NS_IsMainThread());
+  if (mPropertyFlags & PropertyFlags::Continuations) {
+    RemoveProperty(ContinuationsProperty());
+    mPropertyFlags &= ~PropertyFlags::Continuations;
+  }
+}
 
 #endif
