@@ -55,7 +55,7 @@ class GfxInfoBase : public nsIGfxInfo,
                                               nsAString& _retval) override;
 
   NS_IMETHOD GetMonitors(JSContext* cx,
-                         JS::MutableHandleValue _retval) override;
+                         JS::MutableHandle<JS::Value> _retval) override;
   NS_IMETHOD GetFailures(nsTArray<int32_t>& indices,
                          nsTArray<nsCString>& failures) override;
   NS_IMETHOD_(void) LogFailure(const nsACString& failure) override;
@@ -68,9 +68,9 @@ class GfxInfoBase : public nsIGfxInfo,
   NS_IMETHOD GetAzureCanvasBackend(nsAString& aBackend) override;
   NS_IMETHOD GetAzureContentBackend(nsAString& aBackend) override;
   NS_IMETHOD GetUsingGPUProcess(bool* aOutValue) override;
-  NS_IMETHOD GetWebRenderEnabled(bool* aWebRenderEnabled) override;
   NS_IMETHOD GetIsHeadless(bool* aIsHeadless) override;
   NS_IMETHOD GetTargetFrameRate(uint32_t* aTargetFrameRate) override;
+  NS_IMETHOD GetCodecSupportInfo(nsACString& aCodecSupportInfo) override;
 
   // Non-XPCOM method to get IPC data:
   nsTArray<mozilla::gfx::GfxInfoFeatureStatus> GetAllFeatures();
@@ -106,7 +106,7 @@ class GfxInfoBase : public nsIGfxInfo,
   // Convenience to get the application version
   static const nsCString& GetApplicationVersion();
 
-  virtual nsresult FindMonitors(JSContext* cx, JS::HandleObject array);
+  virtual nsresult FindMonitors(JSContext* cx, JS::Handle<JSObject*> array);
 
   static void SetFeatureStatus(
       nsTArray<mozilla::gfx::GfxInfoFeatureStatus>&& aFS);
@@ -114,6 +114,8 @@ class GfxInfoBase : public nsIGfxInfo,
  protected:
   virtual ~GfxInfoBase();
 
+  virtual OperatingSystem GetOperatingSystem() = 0;
+
   virtual nsresult GetFeatureStatusImpl(
       int32_t aFeature, int32_t* aStatus, nsAString& aSuggestedDriverVersion,
       const nsTArray<GfxDriverInfo>& aDriverInfo, nsACString& aFailureId,
@@ -125,9 +127,6 @@ class GfxInfoBase : public nsIGfxInfo,
 
   virtual void DescribeFeatures(JSContext* aCx, JS::Handle<JSObject*> obj);
 
-  bool DoesDesktopEnvironmentMatch(const nsAString& aBlocklistDesktop,
-                                   const nsAString& aDesktopEnv);
-
   virtual bool DoesWindowProtocolMatch(
       const nsAString& aBlocklistWindowProtocol,
       const nsAString& aWindowProtocol);
