@@ -9,10 +9,8 @@
 #include "nsCOMPtr.h"
 #include "nsReadableUtils.h"
 #include "nsNetUtil.h"
-#include "nsIComponentManager.h"
 #include "nsIFile.h"
 #include "nsILocalFileMac.h"
-#include "nsIURL.h"
 #include "nsArrayEnumerator.h"
 #include "nsIStringBundle.h"
 #include "nsCocoaUtils.h"
@@ -24,7 +22,7 @@
 using namespace mozilla;
 
 const float kAccessoryViewPadding = 5;
-const int   kSaveTypeControlTag = 1;
+const int kSaveTypeControlTag = 1;
 
 static bool gCallSecretHiddenFileAPI = false;
 const char kShowHiddenFilesPref[] = "filepicker.showHiddenFiles";
@@ -32,16 +30,15 @@ const char kShowHiddenFilesPref[] = "filepicker.showHiddenFiles";
 /**
  * This class is an observer of NSPopUpButton selection change.
  */
-@interface NSPopUpButtonObserver : NSObject
-{
+@interface NSPopUpButtonObserver : NSObject {
   NSPopUpButton* mPopUpButton;
-  NSOpenPanel*   mOpenPanel;
-  nsFilePicker*  mFilePicker;
+  NSOpenPanel* mOpenPanel;
+  nsFilePicker* mFilePicker;
 }
-- (void) setPopUpButton:(NSPopUpButton*)aPopUpButton;
-- (void) setOpenPanel:(NSOpenPanel*)aOpenPanel;
-- (void) setFilePicker:(nsFilePicker*)aFilePicker;
-- (void) menuChangedItem:(NSNotification*)aSender;
+- (void)setPopUpButton:(NSPopUpButton*)aPopUpButton;
+- (void)setOpenPanel:(NSOpenPanel*)aOpenPanel;
+- (void)setFilePicker:(nsFilePicker*)aFilePicker;
+- (void)menuChangedItem:(NSNotification*)aSender;
 @end
 
 NS_IMPL_ISUPPORTS(nsFilePicker, nsIFilePicker)
@@ -51,9 +48,8 @@ NS_IMPL_ISUPPORTS(nsFilePicker, nsIFilePicker)
 // if it disappears so that we stop showing hidden files if a user deletes
 // the pref. If the secret API was used once and things worked out it should
 // continue working for subsequent calls so the user is at no more risk.
-static void SetShowHiddenFileState(NSSavePanel* panel)
-{
-  NS_OBJC_BEGIN_TRY_ABORT_BLOCK;
+static void SetShowHiddenFileState(NSSavePanel* panel) {
+  NS_OBJC_BEGIN_TRY_IGNORE_BLOCK;
 
   bool show = false;
   if (NS_SUCCEEDED(Preferences::GetBool(kShowHiddenFilesPref, &show))) {
@@ -64,8 +60,7 @@ static void SetShowHiddenFileState(NSSavePanel* panel)
     // invoke a method to get a Cocoa-internal nav view
     SEL navViewSelector = @selector(_navView);
     NSMethodSignature* navViewSignature = [panel methodSignatureForSelector:navViewSelector];
-    if (!navViewSignature)
-      return;
+    if (!navViewSignature) return;
     NSInvocation* navViewInvocation = [NSInvocation invocationWithMethodSignature:navViewSignature];
     [navViewInvocation setSelector:navViewSelector];
     [navViewInvocation setTarget:panel];
@@ -77,37 +72,28 @@ static void SetShowHiddenFileState(NSSavePanel* panel)
 
     // invoke the secret show hidden file state method on the nav view
     SEL showHiddenFilesSelector = @selector(setShowsHiddenFiles:);
-    NSMethodSignature* showHiddenFilesSignature = [navView methodSignatureForSelector:showHiddenFilesSelector];
-    if (!showHiddenFilesSignature)
-      return;
-    NSInvocation* showHiddenFilesInvocation = [NSInvocation invocationWithMethodSignature:showHiddenFilesSignature];
+    NSMethodSignature* showHiddenFilesSignature =
+        [navView methodSignatureForSelector:showHiddenFilesSelector];
+    if (!showHiddenFilesSignature) return;
+    NSInvocation* showHiddenFilesInvocation =
+        [NSInvocation invocationWithMethodSignature:showHiddenFilesSignature];
     [showHiddenFilesInvocation setSelector:showHiddenFilesSelector];
     [showHiddenFilesInvocation setTarget:navView];
     [showHiddenFilesInvocation setArgument:&show atIndex:2];
     [showHiddenFilesInvocation invoke];
   }
 
-  NS_OBJC_END_TRY_ABORT_BLOCK;
+  NS_OBJC_END_TRY_IGNORE_BLOCK;
 }
 
-nsFilePicker::nsFilePicker()
-: mSelectedTypeIndex(0)
-{
-}
+nsFilePicker::nsFilePicker() : mSelectedTypeIndex(0) {}
 
-nsFilePicker::~nsFilePicker()
-{
-}
+nsFilePicker::~nsFilePicker() {}
 
-void
-nsFilePicker::InitNative(nsIWidget *aParent, const nsAString& aTitle)
-{
-  mTitle = aTitle;
-}
+void nsFilePicker::InitNative(nsIWidget* aParent, const nsAString& aTitle) { mTitle = aTitle; }
 
-NSView* nsFilePicker::GetAccessoryView()
-{
-  NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NIL;
+NSView* nsFilePicker::GetAccessoryView() {
+  NS_OBJC_BEGIN_TRY_BLOCK_RETURN;
 
   NSView* accessoryView = [[[NSView alloc] initWithFrame:NSMakeRect(0, 0, 0, 0)] autorelease];
 
@@ -117,7 +103,8 @@ NSView* nsFilePicker::GetAccessoryView()
   // Try to get the localized string.
   nsCOMPtr<nsIStringBundleService> sbs = do_GetService(NS_STRINGBUNDLE_CONTRACTID);
   nsCOMPtr<nsIStringBundle> bundle;
-  nsresult rv = sbs->CreateBundle("chrome://global/locale/filepicker.properties", getter_AddRefs(bundle));
+  nsresult rv =
+      sbs->CreateBundle("chrome://global/locale/filepicker.properties", getter_AddRefs(bundle));
   if (NS_SUCCEEDED(rv)) {
     nsAutoString locaLabel;
     rv = bundle->GetStringFromName("formatLabel", locaLabel);
@@ -140,19 +127,21 @@ NSView* nsFilePicker::GetAccessoryView()
   [textField sizeToFit];
 
   // set up popup button
-  NSPopUpButton* popupButton = [[[NSPopUpButton alloc] initWithFrame:NSMakeRect(0, 0, 0, 0) pullsDown:NO] autorelease];
+  NSPopUpButton* popupButton = [[[NSPopUpButton alloc] initWithFrame:NSMakeRect(0, 0, 0, 0)
+                                                           pullsDown:NO] autorelease];
   uint32_t numMenuItems = mTitles.Length();
   for (uint32_t i = 0; i < numMenuItems; i++) {
     const nsString& currentTitle = mTitles[i];
-    NSString *titleString;
+    NSString* titleString;
     if (currentTitle.IsEmpty()) {
       const nsString& currentFilter = mFilters[i];
-      titleString = [[NSString alloc] initWithCharacters:reinterpret_cast<const unichar*>(currentFilter.get())
-                                                  length:currentFilter.Length()];
-    }
-    else {
-      titleString = [[NSString alloc] initWithCharacters:reinterpret_cast<const unichar*>(currentTitle.get())
-                                                  length:currentTitle.Length()];
+      titleString =
+          [[NSString alloc] initWithCharacters:reinterpret_cast<const unichar*>(currentFilter.get())
+                                        length:currentFilter.Length()];
+    } else {
+      titleString =
+          [[NSString alloc] initWithCharacters:reinterpret_cast<const unichar*>(currentTitle.get())
+                                        length:currentTitle.Length()];
     }
     [popupButton addItemWithTitle:titleString];
     [titleString release];
@@ -160,7 +149,7 @@ NSView* nsFilePicker::GetAccessoryView()
   if (mSelectedTypeIndex >= 0 && (uint32_t)mSelectedTypeIndex < numMenuItems)
     [popupButton selectItemAtIndex:mSelectedTypeIndex];
   [popupButton setTag:kSaveTypeControlTag];
-  [popupButton sizeToFit]; // we have to do sizeToFit to get the height calculated for us
+  [popupButton sizeToFit];  // we have to do sizeToFit to get the height calculated for us
   // This is just a default width that works well, doesn't truncate the vast majority of
   // things that might end up in the menu.
   [popupButton setFrameSize:NSMakeSize(180, [popupButton frame].size.height)];
@@ -171,78 +160,68 @@ NSView* nsFilePicker::GetAccessoryView()
   if ([popupButton frame].size.height > greatestHeight)
     greatestHeight = [popupButton frame].size.height;
   float totalViewHeight = greatestHeight + kAccessoryViewPadding * 2;
-  float totalViewWidth  = [textField frame].size.width + [popupButton frame].size.width + kAccessoryViewPadding * 3;
+  float totalViewWidth =
+      [textField frame].size.width + [popupButton frame].size.width + kAccessoryViewPadding * 3;
   [accessoryView setFrameSize:NSMakeSize(totalViewWidth, totalViewHeight)];
 
-  float textFieldOriginY = ((greatestHeight - [textField frame].size.height) / 2 + 1) + kAccessoryViewPadding;
+  float textFieldOriginY =
+      ((greatestHeight - [textField frame].size.height) / 2 + 1) + kAccessoryViewPadding;
   [textField setFrameOrigin:NSMakePoint(kAccessoryViewPadding, textFieldOriginY)];
-  
+
   float popupOriginX = [textField frame].size.width + kAccessoryViewPadding * 2;
-  float popupOriginY = ((greatestHeight - [popupButton frame].size.height) / 2) + kAccessoryViewPadding;
+  float popupOriginY =
+      ((greatestHeight - [popupButton frame].size.height) / 2) + kAccessoryViewPadding;
   [popupButton setFrameOrigin:NSMakePoint(popupOriginX, popupOriginY)];
 
   [accessoryView addSubview:textField];
   [accessoryView addSubview:popupButton];
   return accessoryView;
 
-  NS_OBJC_END_TRY_ABORT_BLOCK_NIL;
+  NS_OBJC_END_TRY_BLOCK_RETURN(nil);
 }
 
 // Display the file dialog
-nsresult nsFilePicker::Show(int16_t *retval)
-{
+nsresult nsFilePicker::Show(ResultCode* retval) {
   NS_ENSURE_ARG_POINTER(retval);
 
   *retval = returnCancel;
 
-  int16_t userClicksOK = returnCancel;
-
-// Random questions from DHH:
-//
-// Why do we pass mTitle, mDefault to the functions?  Can GetLocalFile. PutLocalFile,
-// and GetLocalFolder get called someplace else?  It generates a bunch of warnings
-// as it is right now.
-//
-// I think we could easily combine GetLocalFile and GetLocalFolder together, just
-// setting panel pick options based on mMode.  I didn't do it here b/c I wanted to 
-// make this look as much like Carbon nsFilePicker as possible.  
+  ResultCode userClicksOK = returnCancel;
 
   mFiles.Clear();
   nsCOMPtr<nsIFile> theFile;
 
-  switch (mMode)
-  {
+  // Note that GetLocalFolder shares a lot of code with GetLocalFiles.
+  // Could combine the functions and just pass the mode in.
+  switch (mMode) {
     case modeOpen:
-      userClicksOK = GetLocalFiles(mTitle, false, mFiles);
+      userClicksOK = GetLocalFiles(false, mFiles);
       break;
-    
+
     case modeOpenMultiple:
-      userClicksOK = GetLocalFiles(mTitle, true, mFiles);
+      userClicksOK = GetLocalFiles(true, mFiles);
       break;
-      
+
     case modeSave:
-      userClicksOK = PutLocalFile(mTitle, mDefault, getter_AddRefs(theFile));
+      userClicksOK = PutLocalFile(getter_AddRefs(theFile));
       break;
-      
+
     case modeGetFolder:
-      userClicksOK = GetLocalFolder(mTitle, getter_AddRefs(theFile));
+      userClicksOK = GetLocalFolder(getter_AddRefs(theFile));
       break;
-    
+
     default:
       NS_ERROR("Unknown file picker mode");
       break;
   }
 
-  if (theFile)
-    mFiles.AppendObject(theFile);
-  
+  if (theFile) mFiles.AppendObject(theFile);
+
   *retval = userClicksOK;
   return NS_OK;
 }
 
-static
-void UpdatePanelFileTypes(NSOpenPanel* aPanel, NSArray* aFilters)
-{
+static void UpdatePanelFileTypes(NSOpenPanel* aPanel, NSArray* aFilters) {
   // If we show all file types, also "expose" bundles' contents.
   [aPanel setTreatsFilePackagesAsDirectories:!aFilters];
 
@@ -250,24 +229,20 @@ void UpdatePanelFileTypes(NSOpenPanel* aPanel, NSArray* aFilters)
 }
 
 @implementation NSPopUpButtonObserver
-- (void) setPopUpButton:(NSPopUpButton*)aPopUpButton
-{
+- (void)setPopUpButton:(NSPopUpButton*)aPopUpButton {
   mPopUpButton = aPopUpButton;
 }
 
-- (void) setOpenPanel:(NSOpenPanel*)aOpenPanel
-{
+- (void)setOpenPanel:(NSOpenPanel*)aOpenPanel {
   mOpenPanel = aOpenPanel;
 }
 
-- (void) setFilePicker:(nsFilePicker*)aFilePicker
-{
+- (void)setFilePicker:(nsFilePicker*)aFilePicker {
   mFilePicker = aFilePicker;
 }
 
-- (void) menuChangedItem:(NSNotification *)aSender
-{
-  NS_OBJC_BEGIN_TRY_ABORT_BLOCK_RETURN;
+- (void)menuChangedItem:(NSNotification*)aSender {
+  NS_OBJC_BEGIN_TRY_BLOCK_RETURN;
   int32_t selectedItem = [mPopUpButton indexOfSelectedItem];
   if (selectedItem < 0) {
     return;
@@ -276,41 +251,40 @@ void UpdatePanelFileTypes(NSOpenPanel* aPanel, NSArray* aFilters)
   mFilePicker->SetFilterIndex(selectedItem);
   UpdatePanelFileTypes(mOpenPanel, mFilePicker->GetFilterList());
 
-  NS_OBJC_END_TRY_ABORT_BLOCK_RETURN();
+  NS_OBJC_END_TRY_BLOCK_RETURN();
 }
 @end
 
-// Use OpenPanel to do a GetFile. Returns |returnOK| if the user presses OK in the dialog. 
-int16_t
-nsFilePicker::GetLocalFiles(const nsString& inTitle, bool inAllowMultiple, nsCOMArray<nsIFile>& outFiles)
-{
-  NS_OBJC_BEGIN_TRY_ABORT_BLOCK_RETURN;
+// Use OpenPanel to do a GetFile. Returns |returnOK| if the user presses OK in the dialog.
+nsIFilePicker::ResultCode nsFilePicker::GetLocalFiles(bool inAllowMultiple,
+                                                      nsCOMArray<nsIFile>& outFiles) {
+  NS_OBJC_BEGIN_TRY_BLOCK_RETURN;
 
-  int16_t retVal = (int16_t)returnCancel;
-  NSOpenPanel *thePanel = [NSOpenPanel openPanel];
+  ResultCode retVal = nsIFilePicker::returnCancel;
+  NSOpenPanel* thePanel = [NSOpenPanel openPanel];
 
   SetShowHiddenFileState(thePanel);
 
   // Set the options for how the get file dialog will appear
-  SetDialogTitle(inTitle, thePanel);
+  SetDialogTitle(mTitle, thePanel);
   [thePanel setAllowsMultipleSelection:inAllowMultiple];
   [thePanel setCanSelectHiddenExtension:YES];
   [thePanel setCanChooseDirectories:NO];
   [thePanel setCanChooseFiles:YES];
-  [thePanel setResolvesAliases:YES]; //this is default - probably doesn't need to be set
-  
+  [thePanel setResolvesAliases:YES];
+
   // Get filters
   // filters may be null, if we should allow all file types.
-  NSArray *filters = GetFilterList();
+  NSArray* filters = GetFilterList();
 
   // set up default directory
-  NSString *theDir = PanelDefaultDirectory();
-  
+  NSString* theDir = PanelDefaultDirectory();
+
   // if this is the "Choose application..." dialog, and no other start
   // dir has been set, then use the Applications folder.
   if (!theDir) {
     if (filters && [filters count] == 1 &&
-        [(NSString *)[filters objectAtIndex:0] isEqualToString:@"app"])
+        [(NSString*)[filters objectAtIndex:0] isEqualToString:@"app"])
       theDir = @"/Applications/";
     else
       theDir = @"";
@@ -334,10 +308,10 @@ nsFilePicker::GetLocalFiles(const nsString& inTitle, bool inAllowMultiple, nsCOM
     [observer setOpenPanel:thePanel];
     [observer setFilePicker:this];
 
-    [[NSNotificationCenter defaultCenter]
-      addObserver:observer
-      selector:@selector(menuChangedItem:)
-      name:NSMenuWillSendActionNotification object:nil];
+    [[NSNotificationCenter defaultCenter] addObserver:observer
+                                             selector:@selector(menuChangedItem:)
+                                                 name:NSMenuWillSendActionNotification
+                                               object:nil];
 
     UpdatePanelFileTypes(thePanel, filters);
     result = [thePanel runModal];
@@ -353,9 +327,8 @@ nsFilePicker::GetLocalFiles(const nsString& inTitle, bool inAllowMultiple, nsCOM
     result = [thePanel runModal];
   }
   nsCocoaUtils::CleanUpAfterNativeAppModalDialog();
-  
-  if (result == NSFileHandlingPanelCancelButton)
-    return retVal;
+
+  if (result == NSFileHandlingPanelCancelButton) return retVal;
 
   // Converts data from a NSArray of NSURL to the returned format.
   // We should be careful to not call [thePanel URLs] more than once given that
@@ -368,47 +341,44 @@ nsFilePicker::GetLocalFiles(const nsString& inTitle, bool inAllowMultiple, nsCOM
     }
 
     nsCOMPtr<nsIFile> localFile;
-    NS_NewLocalFile(EmptyString(), true, getter_AddRefs(localFile));
+    NS_NewLocalFile(u""_ns, true, getter_AddRefs(localFile));
     nsCOMPtr<nsILocalFileMac> macLocalFile = do_QueryInterface(localFile);
     if (macLocalFile && NS_SUCCEEDED(macLocalFile->InitWithCFURL((CFURLRef)url))) {
       outFiles.AppendObject(localFile);
     }
   }
 
-  if (outFiles.Count() > 0)
-    retVal = returnOK;
+  if (outFiles.Count() > 0) retVal = returnOK;
 
   return retVal;
 
-  NS_OBJC_END_TRY_ABORT_BLOCK_RETURN(0);
+  NS_OBJC_END_TRY_BLOCK_RETURN(nsIFilePicker::returnOK);
 }
 
 // Use OpenPanel to do a GetFolder. Returns |returnOK| if the user presses OK in the dialog.
-int16_t
-nsFilePicker::GetLocalFolder(const nsString& inTitle, nsIFile** outFile)
-{
-  NS_OBJC_BEGIN_TRY_ABORT_BLOCK_RETURN;
+nsIFilePicker::ResultCode nsFilePicker::GetLocalFolder(nsIFile** outFile) {
+  NS_OBJC_BEGIN_TRY_BLOCK_RETURN;
   NS_ASSERTION(outFile, "this protected member function expects a null initialized out pointer");
-  
-  int16_t retVal = (int16_t)returnCancel;
-  NSOpenPanel *thePanel = [NSOpenPanel openPanel];
+
+  ResultCode retVal = nsIFilePicker::returnCancel;
+  NSOpenPanel* thePanel = [NSOpenPanel openPanel];
 
   SetShowHiddenFileState(thePanel);
 
   // Set the options for how the get file dialog will appear
-  SetDialogTitle(inTitle, thePanel);
-  [thePanel setAllowsMultipleSelection:NO];   //this is default -probably doesn't need to be set
+  SetDialogTitle(mTitle, thePanel);
+  [thePanel setAllowsMultipleSelection:NO];
   [thePanel setCanSelectHiddenExtension:YES];
   [thePanel setCanChooseDirectories:YES];
   [thePanel setCanChooseFiles:NO];
-  [thePanel setResolvesAliases:YES];          //this is default - probably doesn't need to be set
+  [thePanel setResolvesAliases:YES];
   [thePanel setCanCreateDirectories:YES];
-  
+
   // packages != folders
   [thePanel setTreatsFilePackagesAsDirectories:NO];
 
   // set up default directory
-  NSString *theDir = PanelDefaultDirectory();
+  NSString* theDir = PanelDefaultDirectory();
   if (theDir) {
     [thePanel setDirectoryURL:[NSURL fileURLWithPath:theDir isDirectory:YES]];
   }
@@ -416,14 +386,13 @@ nsFilePicker::GetLocalFolder(const nsString& inTitle, nsIFile** outFile)
   int result = [thePanel runModal];
   nsCocoaUtils::CleanUpAfterNativeAppModalDialog();
 
-  if (result == NSFileHandlingPanelCancelButton)
-    return retVal;
+  if (result == NSFileHandlingPanelCancelButton) return retVal;
 
   // get the path for the folder (we allow just 1, so that's all we get)
-  NSURL *theURL = [[thePanel URLs] objectAtIndex:0];
+  NSURL* theURL = [[thePanel URLs] objectAtIndex:0];
   if (theURL) {
     nsCOMPtr<nsIFile> localFile;
-    NS_NewLocalFile(EmptyString(), true, getter_AddRefs(localFile));
+    NS_NewLocalFile(u""_ns, true, getter_AddRefs(localFile));
     nsCOMPtr<nsILocalFileMac> macLocalFile = do_QueryInterface(localFile);
     if (macLocalFile && NS_SUCCEEDED(macLocalFile->InitWithCFURL((CFURLRef)theURL))) {
       *outFile = localFile;
@@ -434,34 +403,33 @@ nsFilePicker::GetLocalFolder(const nsString& inTitle, nsIFile** outFile)
 
   return retVal;
 
-  NS_OBJC_END_TRY_ABORT_BLOCK_RETURN(0);
+  NS_OBJC_END_TRY_BLOCK_RETURN(nsIFilePicker::returnOK);
 }
 
 // Returns |returnOK| if the user presses OK in the dialog.
-int16_t
-nsFilePicker::PutLocalFile(const nsString& inTitle, const nsString& inDefaultName, nsIFile** outFile)
-{
-  NS_OBJC_BEGIN_TRY_ABORT_BLOCK_RETURN;
+nsIFilePicker::ResultCode nsFilePicker::PutLocalFile(nsIFile** outFile) {
+  NS_OBJC_BEGIN_TRY_BLOCK_RETURN;
   NS_ASSERTION(outFile, "this protected member function expects a null initialized out pointer");
 
-  int16_t retVal = returnCancel;
-  NSSavePanel *thePanel = [NSSavePanel savePanel];
+  ResultCode retVal = nsIFilePicker::returnCancel;
+  NSSavePanel* thePanel = [NSSavePanel savePanel];
 
   SetShowHiddenFileState(thePanel);
 
-  SetDialogTitle(inTitle, thePanel);
+  SetDialogTitle(mTitle, thePanel);
 
   // set up accessory view for file format options
   NSView* accessoryView = GetAccessoryView();
   [thePanel setAccessoryView:accessoryView];
 
   // set up default file name
-  NSString* defaultFilename = [NSString stringWithCharacters:(const unichar*)inDefaultName.get() length:inDefaultName.Length()];
+  NSString* defaultFilename = [NSString stringWithCharacters:(const unichar*)mDefaultFilename.get()
+                                                      length:mDefaultFilename.Length()];
 
   // Set up the allowed type. This prevents the extension from being selected.
   NSString* extension = defaultFilename.pathExtension;
   if (extension.length != 0) {
-    thePanel.allowedFileTypes = @[extension];
+    thePanel.allowedFileTypes = @[ extension ];
   }
   // Allow users to change the extension.
   thePanel.allowsOtherFileTypes = YES;
@@ -472,14 +440,12 @@ nsFilePicker::PutLocalFile(const nsString& inTitle, const nsString& inDefaultNam
   // trying to add a non-default extension. To avoid the confusion, we ensure
   // that all extensions are shown in the panel if the remaining extension is
   // known by the OS.
-  NSString* fileName =
-    [[defaultFilename lastPathComponent] stringByDeletingPathExtension];
+  NSString* fileName = [[defaultFilename lastPathComponent] stringByDeletingPathExtension];
   NSString* otherExtension = fileName.pathExtension;
   if (otherExtension.length != 0) {
     // There's another extension here. Get the UTI.
-    CFStringRef type =
-      UTTypeCreatePreferredIdentifierForTag(kUTTagClassFilenameExtension,
-                                            (CFStringRef)otherExtension, NULL);
+    CFStringRef type = UTTypeCreatePreferredIdentifierForTag(kUTTagClassFilenameExtension,
+                                                             (CFStringRef)otherExtension, NULL);
     if (type) {
       if (!CFStringHasPrefix(type, CFSTR("dyn."))) {
         // We have a UTI, otherwise the type would have a "dyn." prefix. Ensure
@@ -491,7 +457,7 @@ nsFilePicker::PutLocalFile(const nsString& inTitle, const nsString& inDefaultNam
   }
 
   // set up default directory
-  NSString *theDir = PanelDefaultDirectory();
+  NSString* theDir = PanelDefaultDirectory();
   if (theDir) {
     [thePanel setDirectoryURL:[NSURL fileURLWithPath:theDir isDirectory:YES]];
   }
@@ -501,8 +467,7 @@ nsFilePicker::PutLocalFile(const nsString& inTitle, const nsString& inDefaultNam
   [thePanel setNameFieldStringValue:defaultFilename];
   int result = [thePanel runModal];
   nsCocoaUtils::CleanUpAfterNativeAppModalDialog();
-  if (result == NSFileHandlingPanelCancelButton)
-    return retVal;
+  if (result == NSFileHandlingPanelCancelButton) return retVal;
 
   // get the save type
   NSPopUpButton* popupButton = [accessoryView viewWithTag:kSaveTypeControlTag];
@@ -511,9 +476,9 @@ nsFilePicker::PutLocalFile(const nsString& inTitle, const nsString& inDefaultNam
   }
 
   NSURL* fileURL = [thePanel URL];
-  if (fileURL) { 
+  if (fileURL) {
     nsCOMPtr<nsIFile> localFile;
-    NS_NewLocalFile(EmptyString(), true, getter_AddRefs(localFile));
+    NS_NewLocalFile(u""_ns, true, getter_AddRefs(localFile));
     nsCOMPtr<nsILocalFileMac> macLocalFile = do_QueryInterface(localFile);
     if (macLocalFile && NS_SUCCEEDED(macLocalFile->InitWithCFURL((CFURLRef)fileURL))) {
       *outFile = localFile;
@@ -529,13 +494,11 @@ nsFilePicker::PutLocalFile(const nsString& inTitle, const nsString& inDefaultNam
 
   return retVal;
 
-  NS_OBJC_END_TRY_ABORT_BLOCK_RETURN(0);
+  NS_OBJC_END_TRY_BLOCK_RETURN(nsIFilePicker::returnCancel);
 }
 
-NSArray *
-nsFilePicker::GetFilterList()
-{
-  NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NIL;
+NSArray* nsFilePicker::GetFilterList() {
+  NS_OBJC_BEGIN_TRY_BLOCK_RETURN;
 
   if (!mFilters.Length()) {
     return nil;
@@ -551,68 +514,67 @@ nsFilePicker::GetFilterList()
     return nil;
   }
 
-  if (filterWide.Equals(NS_LITERAL_STRING("*"))) {
+  if (filterWide.Equals(u"*"_ns)) {
     return nil;
   }
 
   // The extensions in filterWide are in the format "*.ext" but are expected
   // in the format "ext" by NSOpenPanel. So we need to filter some characters.
-  NSMutableString* filterString = [[[NSMutableString alloc] initWithString:
-                                    [NSString stringWithCharacters:reinterpret_cast<const unichar*>(filterWide.get())
-                                                            length:filterWide.Length()]] autorelease];
-  NSCharacterSet *set = [NSCharacterSet characterSetWithCharactersInString:@". *"];
+  NSMutableString* filterString = [[[NSMutableString alloc]
+      initWithString:[NSString
+                         stringWithCharacters:reinterpret_cast<const unichar*>(filterWide.get())
+                                       length:filterWide.Length()]] autorelease];
+  NSCharacterSet* set = [NSCharacterSet characterSetWithCharactersInString:@". *"];
   NSRange range = [filterString rangeOfCharacterFromSet:set];
   while (range.length) {
     [filterString replaceCharactersInRange:range withString:@""];
     range = [filterString rangeOfCharacterFromSet:set];
   }
 
-  return [[[NSArray alloc] initWithArray:
-           [filterString componentsSeparatedByString:@";"]] autorelease];
+  return
+      [[[NSArray alloc] initWithArray:[filterString componentsSeparatedByString:@";"]] autorelease];
 
-  NS_OBJC_END_TRY_ABORT_BLOCK_NIL;
+  NS_OBJC_END_TRY_BLOCK_RETURN(nil);
 }
 
 // Sets the dialog title to whatever it should be.  If it fails, eh,
 // the OS will provide a sensible default.
-void
-nsFilePicker::SetDialogTitle(const nsString& inTitle, id aPanel)
-{
-  NS_OBJC_BEGIN_TRY_ABORT_BLOCK;
+void nsFilePicker::SetDialogTitle(const nsString& inTitle, id aPanel) {
+  NS_OBJC_BEGIN_TRY_IGNORE_BLOCK;
 
-  [aPanel setTitle:[NSString stringWithCharacters:(const unichar*)inTitle.get() length:inTitle.Length()]];
+  [aPanel setTitle:[NSString stringWithCharacters:(const unichar*)inTitle.get()
+                                           length:inTitle.Length()]];
 
   if (!mOkButtonLabel.IsEmpty()) {
-    [aPanel setPrompt:[NSString stringWithCharacters:(const unichar*)mOkButtonLabel.get() length:mOkButtonLabel.Length()]];
+    [aPanel setPrompt:[NSString stringWithCharacters:(const unichar*)mOkButtonLabel.get()
+                                              length:mOkButtonLabel.Length()]];
   }
 
-  NS_OBJC_END_TRY_ABORT_BLOCK;
-} 
+  NS_OBJC_END_TRY_IGNORE_BLOCK;
+}
 
 // Converts path from an nsIFile into a NSString path
 // If it fails, returns an empty string.
-NSString *
-nsFilePicker::PanelDefaultDirectory()
-{
-  NS_OBJC_BEGIN_TRY_ABORT_BLOCK_NIL;
+NSString* nsFilePicker::PanelDefaultDirectory() {
+  NS_OBJC_BEGIN_TRY_BLOCK_RETURN;
 
-  NSString *directory = nil;
+  NSString* directory = nil;
   if (mDisplayDirectory) {
     nsAutoString pathStr;
     mDisplayDirectory->GetPath(pathStr);
-    directory = [[[NSString alloc] initWithCharacters:reinterpret_cast<const unichar*>(pathStr.get())
-                                               length:pathStr.Length()] autorelease];
+    directory =
+        [[[NSString alloc] initWithCharacters:reinterpret_cast<const unichar*>(pathStr.get())
+                                       length:pathStr.Length()] autorelease];
   }
   return directory;
 
-  NS_OBJC_END_TRY_ABORT_BLOCK_NIL;
+  NS_OBJC_END_TRY_BLOCK_RETURN(nil);
 }
 
-NS_IMETHODIMP nsFilePicker::GetFile(nsIFile **aFile)
-{
+NS_IMETHODIMP nsFilePicker::GetFile(nsIFile** aFile) {
   NS_ENSURE_ARG_POINTER(aFile);
   *aFile = nullptr;
-  
+
   // just return the first file
   if (mFiles.Count() > 0) {
     *aFile = mFiles.ObjectAt(0);
@@ -622,70 +584,56 @@ NS_IMETHODIMP nsFilePicker::GetFile(nsIFile **aFile)
   return NS_OK;
 }
 
-NS_IMETHODIMP nsFilePicker::GetFileURL(nsIURI **aFileURL)
-{
+NS_IMETHODIMP nsFilePicker::GetFileURL(nsIURI** aFileURL) {
   NS_ENSURE_ARG_POINTER(aFileURL);
   *aFileURL = nullptr;
 
-  if (mFiles.Count() == 0)
-    return NS_OK;
+  if (mFiles.Count() == 0) return NS_OK;
 
   return NS_NewFileURI(aFileURL, mFiles.ObjectAt(0));
 }
 
-NS_IMETHODIMP nsFilePicker::GetFiles(nsISimpleEnumerator **aFiles)
-{
-  return NS_NewArrayEnumerator(aFiles, mFiles);
+NS_IMETHODIMP nsFilePicker::GetFiles(nsISimpleEnumerator** aFiles) {
+  return NS_NewArrayEnumerator(aFiles, mFiles, NS_GET_IID(nsIFile));
 }
 
-NS_IMETHODIMP nsFilePicker::SetDefaultString(const nsAString& aString)
-{
-  mDefault = aString;
+NS_IMETHODIMP nsFilePicker::SetDefaultString(const nsAString& aString) {
+  mDefaultFilename = aString;
   return NS_OK;
 }
 
-NS_IMETHODIMP nsFilePicker::GetDefaultString(nsAString& aString)
-{
-  return NS_ERROR_FAILURE;
-}
+NS_IMETHODIMP nsFilePicker::GetDefaultString(nsAString& aString) { return NS_ERROR_FAILURE; }
 
 // The default extension to use for files
-NS_IMETHODIMP nsFilePicker::GetDefaultExtension(nsAString& aExtension)
-{
+NS_IMETHODIMP nsFilePicker::GetDefaultExtension(nsAString& aExtension) {
   aExtension.Truncate();
   return NS_OK;
 }
 
-NS_IMETHODIMP nsFilePicker::SetDefaultExtension(const nsAString& aExtension)
-{
-  return NS_OK;
-}
+NS_IMETHODIMP nsFilePicker::SetDefaultExtension(const nsAString& aExtension) { return NS_OK; }
 
 // Append an entry to the filters array
 NS_IMETHODIMP
-nsFilePicker::AppendFilter(const nsAString& aTitle, const nsAString& aFilter)
-{
+nsFilePicker::AppendFilter(const nsAString& aTitle, const nsAString& aFilter) {
   // "..apps" has to be translated with native executable extensions.
   if (aFilter.EqualsLiteral("..apps")) {
-    mFilters.AppendElement(NS_LITERAL_STRING("*.app"));
+    mFilters.AppendElement(u"*.app"_ns);
   } else {
     mFilters.AppendElement(aFilter);
   }
   mTitles.AppendElement(aTitle);
-  
+
   return NS_OK;
 }
 
 // Get the filter index - do we still need this?
-NS_IMETHODIMP nsFilePicker::GetFilterIndex(int32_t *aFilterIndex)
-{
+NS_IMETHODIMP nsFilePicker::GetFilterIndex(int32_t* aFilterIndex) {
   *aFilterIndex = mSelectedTypeIndex;
   return NS_OK;
 }
 
 // Set the filter index - do we still need this?
-NS_IMETHODIMP nsFilePicker::SetFilterIndex(int32_t aFilterIndex)
-{
+NS_IMETHODIMP nsFilePicker::SetFilterIndex(int32_t aFilterIndex) {
   mSelectedTypeIndex = aFilterIndex;
   return NS_OK;
 }
