@@ -10,6 +10,7 @@
 #include <shlwapi.h>
 #include <cderr.h>
 
+#include "mozilla/Assertions.h"
 #include "mozilla/BackgroundHangMonitor.h"
 #include "mozilla/ProfilerLabels.h"
 #include "mozilla/UniquePtr.h"
@@ -24,8 +25,8 @@
 #include "WinUtils.h"
 #include "nsPIDOMWindow.h"
 
-using mozilla::IsWin8OrLater;
-using mozilla::MakeUnique;
+#include "mozilla/widget/filedialog/WinFileDialogCommands.h"
+
 using mozilla::UniquePtr;
 
 using namespace mozilla::widget;
@@ -34,30 +35,10 @@ UniquePtr<char16_t[], nsFilePicker::FreeDeleter>
     nsFilePicker::sLastUsedUnicodeDirectory;
 
 #define MAX_EXTENSION_LENGTH 10
-#define FILE_BUFFER_SIZE 4096
-
-typedef DWORD FILEOPENDIALOGOPTIONS;
 
 ///////////////////////////////////////////////////////////////////////////////
 // Helper classes
 
-// Manages NS_NATIVE_TMP_WINDOW child windows. NS_NATIVE_TMP_WINDOWs are
-// temporary child windows of mParentWidget created to address RTL issues
-// in picker dialogs. We are responsible for destroying these.
-class AutoDestroyTmpWindow {
- public:
-  explicit AutoDestroyTmpWindow(HWND aTmpWnd) : mWnd(aTmpWnd) {}
-
-  ~AutoDestroyTmpWindow() {
-    if (mWnd) DestroyWindow(mWnd);
-  }
-
-  inline HWND get() const { return mWnd; }
-
- private:
-  HWND mWnd;
-};
-
 // Manages matching PickerOpen/PickerClosed calls on the parent widget.
 class AutoWidgetPickerState {
  public:
@@ -88,7 +69,8 @@ nsFilePicker::nsFilePicker() : mSelectedType(1) {}
 NS_IMPL_ISUPPORTS(nsFilePicker, nsIFilePicker)
 
 NS_IMETHODIMP nsFilePicker::Init(mozIDOMWindowProxy* aParent,
-                                 const nsAString& aTitle, int16_t aMode) {
+                                 const nsAString& aTitle,
+                                 nsIFilePicker::Mode aMode) {
   nsCOMPtr<nsPIDOMWindowOuter> window = do_QueryInterface(aParent);
   nsIDocShell* docShell = window ? window->GetDocShell() : nullptr;
   mLoadContext = do_QueryInterface(docShell);
@@ -115,70 +97,40 @@ bool nsFilePicker::ShowFolderPicker(const nsString& aInitialDir) {
     return false;
   }
 
-  // options
-  FILEOPENDIALOGOPTIONS fos = FOS_PICKFOLDERS;
-  HRESULT hr = dialog->SetOptions(fos);
-  if (FAILED(hr)) {
-    return false;
-  }
-
-  // initial strings
-  hr = dialog->SetTitle(mTitle.get());
-  if (FAILED(hr)) {
-    return false;
-  }
+  namespace fd = ::mozilla::widget::filedialog;
+  nsTArray<fd::Command> commands = {
+      fd::SetOptions(FOS_PICKFOLDERS),
+      fd::SetTitle(mTitle),
+  };
 
   if (!mOkButtonLabel.IsEmpty()) {
-    hr = dialog->SetOkButtonLabel(mOkButtonLabel.get());
-    if (FAILED(hr)) {
+    commands.AppendElement(fd::SetOkButtonLabel(mOkButtonLabel));
+  }
+
+  if (!aInitialDir.IsEmpty()) {
+    commands.AppendElement(fd::SetFolder(aInitialDir));
+  }
+
+  {
+    if (NS_FAILED(fd::ApplyCommands(dialog, commands))) {
+      return false;
+    }
+
+    ScopedRtlShimWindow shim(mParentWidget.get());
+    mozilla::BackgroundHangMonitor().NotifyWait();
+
+    if (FAILED(dialog->Show(shim.get()))) {
       return false;
     }
   }
 
-  if (!aInitialDir.IsEmpty()) {
-    RefPtr<IShellItem> folder;
-    if (SUCCEEDED(SHCreateItemFromParsingName(aInitialDir.get(), nullptr,
-                                              IID_IShellItem,
-                                              getter_AddRefs(folder)))) {
-      hr = dialog->SetFolder(folder);
-      if (FAILED(hr)) {
-        return false;
-      }
-    }
-  }
-
-  AutoDestroyTmpWindow adtw(
-      (HWND)(mParentWidget.get()
-                 ? mParentWidget->GetNativeData(NS_NATIVE_TMP_WINDOW)
-                 : nullptr));
-
-  // display
-  mozilla::BackgroundHangMonitor().NotifyWait();
-  RefPtr<IShellItem> item;
-  if (FAILED(dialog->Show(adtw.get())) ||
-      FAILED(dialog->GetResult(getter_AddRefs(item))) || !item) {
+  auto result = fd::GetFolderResults(dialog.get());
+  if (result.isErr()) {
     return false;
   }
 
-  // results
-
-  // If the user chose a Win7 Library, resolve to the library's
-  // default save folder.
-  RefPtr<IShellItem> folderPath;
-  RefPtr<IShellLibrary> shellLib;
-  if (FAILED(CoCreateInstance(CLSID_ShellLibrary, nullptr, CLSCTX_INPROC_SERVER,
-                              IID_IShellLibrary, getter_AddRefs(shellLib)))) {
-    return false;
-  }
-
-  if (shellLib && SUCCEEDED(shellLib->LoadLibraryFromItem(item, STGM_READ)) &&
-      SUCCEEDED(shellLib->GetDefaultSaveFolder(DSFT_DETECT, IID_IShellItem,
-                                               getter_AddRefs(folderPath)))) {
-    item.swap(folderPath);
-  }
-
-  // get the folder's file system path
-  return WinUtils::GetShellItemPath(item, mUnicodeFile);
+  mUnicodeFile = result.unwrap();
+  return true;
 }
 
 /*
@@ -210,148 +162,126 @@ bool nsFilePicker::ShowFilePicker(const nsString& aInitialDir) {
     }
   }
 
+  namespace fd = ::mozilla::widget::filedialog;
+  nsTArray<fd::Command> commands;
   // options
+  {
+    FILEOPENDIALOGOPTIONS fos = 0;
+    fos |= FOS_SHAREAWARE | FOS_OVERWRITEPROMPT | FOS_FORCEFILESYSTEM;
 
-  FILEOPENDIALOGOPTIONS fos = 0;
-  fos |= FOS_SHAREAWARE | FOS_OVERWRITEPROMPT | FOS_FORCEFILESYSTEM;
+    // Handle add to recent docs settings
+    if (IsPrivacyModeEnabled() || !mAddToRecentDocs) {
+      fos |= FOS_DONTADDTORECENT;
+    }
 
-  // Handle add to recent docs settings
-  if (IsPrivacyModeEnabled() || !mAddToRecentDocs) {
-    fos |= FOS_DONTADDTORECENT;
-  }
+    // mode specific
+    switch (mMode) {
+      case modeOpen:
+        fos |= FOS_FILEMUSTEXIST;
+        break;
 
-  // mode specific
-  switch (mMode) {
-    case modeOpen:
-      fos |= FOS_FILEMUSTEXIST;
-      break;
+      case modeOpenMultiple:
+        fos |= FOS_FILEMUSTEXIST | FOS_ALLOWMULTISELECT;
+        break;
 
-    case modeOpenMultiple:
-      fos |= FOS_FILEMUSTEXIST | FOS_ALLOWMULTISELECT;
-      break;
+      case modeSave:
+        fos |= FOS_NOREADONLYRETURN;
+        // Don't follow shortcuts when saving a shortcut, this can be used
+        // to trick users (bug 271732)
+        if (IsDefaultPathLink()) fos |= FOS_NODEREFERENCELINKS;
+        break;
 
-    case modeSave:
-      fos |= FOS_NOREADONLYRETURN;
-      // Don't follow shortcuts when saving a shortcut, this can be used
-      // to trick users (bug 271732)
-      if (IsDefaultPathLink()) fos |= FOS_NODEREFERENCELINKS;
-      break;
-  }
+      case modeGetFolder:
+        MOZ_ASSERT(false, "file-picker opened in directory-picker mode");
+        return false;
+    }
 
-  HRESULT hr = dialog->SetOptions(fos);
-  if (FAILED(hr)) {
-    return false;
+    commands.AppendElement(fd::SetOptions(fos));
   }
 
   // initial strings
 
   // title
-  hr = dialog->SetTitle(mTitle.get());
-  if (FAILED(hr)) {
-    return false;
-  }
+  commands.AppendElement(fd::SetTitle(mTitle));
 
   // default filename
   if (!mDefaultFilename.IsEmpty()) {
-    hr = dialog->SetFileName(mDefaultFilename.get());
-    if (FAILED(hr)) {
-      return false;
-    }
+    // Prevent the shell from expanding environment variables by removing
+    // the % characters that are used to delimit them.
+    nsAutoString sanitizedFilename(mDefaultFilename);
+    sanitizedFilename.ReplaceChar('%', '_');
+
+    commands.AppendElement(fd::SetFileName(sanitizedFilename));
   }
 
   // default extension to append to new files
   if (!mDefaultExtension.IsEmpty()) {
-    hr = dialog->SetDefaultExtension(mDefaultExtension.get());
-    if (FAILED(hr)) {
-      return false;
-    }
+    // We don't want environment variables expanded in the extension either.
+    nsAutoString sanitizedExtension(mDefaultExtension);
+    sanitizedExtension.ReplaceChar('%', '_');
+
+    commands.AppendElement(fd::SetDefaultExtension(sanitizedExtension));
   } else if (IsDefaultPathHtml()) {
-    hr = dialog->SetDefaultExtension(L"html");
-    if (FAILED(hr)) {
-      return false;
-    }
+    commands.AppendElement(fd::SetDefaultExtension(u"html"_ns));
   }
 
   // initial location
   if (!aInitialDir.IsEmpty()) {
-    RefPtr<IShellItem> folder;
-    if (SUCCEEDED(SHCreateItemFromParsingName(aInitialDir.get(), nullptr,
-                                              IID_IShellItem,
-                                              getter_AddRefs(folder)))) {
-      hr = dialog->SetFolder(folder);
-      if (FAILED(hr)) {
-        return false;
-      }
-    }
+    commands.AppendElement(fd::SetFolder(aInitialDir));
   }
 
   // filter types and the default index
-  if (!mComFilterList.IsEmpty()) {
-    hr = dialog->SetFileTypes(mComFilterList.Length(), mComFilterList.get());
-    if (FAILED(hr)) {
-      return false;
-    }
-
-    hr = dialog->SetFileTypeIndex(mSelectedType);
-    if (FAILED(hr)) {
-      return false;
+  if (!mFilterList.IsEmpty()) {
+    nsTArray<fd::ComDlgFilterSpec> fileTypes;
+    for (auto const& filter : mFilterList) {
+      fileTypes.EmplaceBack(filter.title, filter.filter);
     }
+    commands.AppendElement(fd::SetFileTypes(std::move(fileTypes)));
+    commands.AppendElement(fd::SetFileTypeIndex(mSelectedType));
   }
 
   // display
-
   {
-    AutoDestroyTmpWindow adtw(
-        (HWND)(mParentWidget.get()
-                   ? mParentWidget->GetNativeData(NS_NATIVE_TMP_WINDOW)
-                   : nullptr));
+    if (NS_FAILED(fd::ApplyCommands(dialog, commands))) {
+      return false;
+    }
+
+    ScopedRtlShimWindow shim(mParentWidget.get());
     AutoWidgetPickerState awps(mParentWidget);
 
     mozilla::BackgroundHangMonitor().NotifyWait();
-    if (FAILED(dialog->Show(adtw.get()))) {
+    if (FAILED(dialog->Show(shim.get()))) {
       return false;
     }
   }
 
   // results
+  auto result_ = fd::GetFileResults(dialog.get());
+  if (result_.isErr()) {
+    return false;
+  }
+  auto result = result_.unwrap();
 
   // Remember what filter type the user selected
-  UINT filterIdxResult;
-  if (SUCCEEDED(dialog->GetFileTypeIndex(&filterIdxResult))) {
-    mSelectedType = (int16_t)filterIdxResult;
-  }
+  mSelectedType = result.selectedFileTypeIndex();
+
+  auto const& paths = result.paths();
 
   // single selection
   if (mMode != modeOpenMultiple) {
-    RefPtr<IShellItem> item;
-    if (FAILED(dialog->GetResult(getter_AddRefs(item))) || !item) return false;
-    return WinUtils::GetShellItemPath(item, mUnicodeFile);
+    if (!paths.IsEmpty()) {
+      MOZ_ASSERT(paths.Length() == 1);
+      mUnicodeFile = paths[0];
+      return true;
+    }
+    return false;
   }
 
   // multiple selection
-  RefPtr<IFileOpenDialog> openDlg;
-  dialog->QueryInterface(IID_IFileOpenDialog, getter_AddRefs(openDlg));
-  if (!openDlg) {
-    // should not happen
-    return false;
-  }
-
-  RefPtr<IShellItemArray> items;
-  if (FAILED(openDlg->GetResults(getter_AddRefs(items))) || !items) {
-    return false;
-  }
-
-  DWORD count = 0;
-  items->GetCount(&count);
-  for (unsigned int idx = 0; idx < count; idx++) {
-    RefPtr<IShellItem> item;
-    nsAutoString str;
-    if (SUCCEEDED(items->GetItemAt(idx, getter_AddRefs(item)))) {
-      if (!WinUtils::GetShellItemPath(item, str)) continue;
-      nsCOMPtr<nsIFile> file;
-      if (NS_SUCCEEDED(NS_NewLocalFile(str, false, getter_AddRefs(file)))) {
-        mFiles.AppendObject(file);
-      }
+  for (auto const& str : paths) {
+    nsCOMPtr<nsIFile> file;
+    if (NS_SUCCEEDED(NS_NewLocalFile(str, false, getter_AddRefs(file)))) {
+      mFiles.AppendObject(file);
     }
   }
   return true;
@@ -360,7 +290,7 @@ bool nsFilePicker::ShowFilePicker(const nsString& aInitialDir) {
 ///////////////////////////////////////////////////////////////////////////////
 // nsIFilePicker impl.
 
-nsresult nsFilePicker::ShowW(int16_t* aReturnVal) {
+nsresult nsFilePicker::ShowW(nsIFilePicker::ResultCode* aReturnVal) {
   NS_ENSURE_ARG_POINTER(aReturnVal);
 
   *aReturnVal = returnCancel;
@@ -394,7 +324,7 @@ nsresult nsFilePicker::ShowW(int16_t* aReturnVal) {
 
   RememberLastUsedDirectory();
 
-  int16_t retValue = returnOK;
+  nsIFilePicker::ResultCode retValue = returnOK;
   if (mMode == modeSave) {
     // Windows does not return resultReplace, we must check if file
     // already exists.
@@ -411,7 +341,9 @@ nsresult nsFilePicker::ShowW(int16_t* aReturnVal) {
   return NS_OK;
 }
 
-nsresult nsFilePicker::Show(int16_t* aReturnVal) { return ShowW(aReturnVal); }
+nsresult nsFilePicker::Show(nsIFilePicker::ResultCode* aReturnVal) {
+  return ShowW(aReturnVal);
+}
 
 NS_IMETHODIMP
 nsFilePicker::GetFile(nsIFile** aFile) {
@@ -453,7 +385,7 @@ nsBaseWinFilePicker::SetDefaultString(const nsAString& aString) {
 
   // First, make sure the file name is not too long.
   int32_t nameLength;
-  int32_t nameIndex = mDefaultFilePath.RFind("\\");
+  int32_t nameIndex = mDefaultFilePath.RFind(u"\\");
   if (nameIndex == kNotFound)
     nameIndex = 0;
   else
@@ -462,7 +394,7 @@ nsBaseWinFilePicker::SetDefaultString(const nsAString& aString) {
   mDefaultFilename.Assign(Substring(mDefaultFilePath, nameIndex));
 
   if (nameLength > MAX_PATH) {
-    int32_t extIndex = mDefaultFilePath.RFind(".");
+    int32_t extIndex = mDefaultFilePath.RFind(u".");
     if (extIndex == kNotFound) extIndex = mDefaultFilePath.Length();
 
     // Let's try to shave the needed characters from the name part.
@@ -474,8 +406,8 @@ nsBaseWinFilePicker::SetDefaultString(const nsAString& aString) {
 
   // Then, we need to replace illegal characters. At this stage, we cannot
   // replace the backslash as the string might represent a file path.
-  mDefaultFilePath.ReplaceChar(FILE_ILLEGAL_CHARACTERS, '-');
-  mDefaultFilename.ReplaceChar(FILE_ILLEGAL_CHARACTERS, '-');
+  mDefaultFilePath.ReplaceChar(u"" FILE_ILLEGAL_CHARACTERS, u'-');
+  mDefaultFilename.ReplaceChar(u"" FILE_ILLEGAL_CHARACTERS, u'-');
 
   return NS_OK;
 }
@@ -520,7 +452,19 @@ void nsFilePicker::InitNative(nsIWidget* aParent, const nsAString& aTitle) {
 
 NS_IMETHODIMP
 nsFilePicker::AppendFilter(const nsAString& aTitle, const nsAString& aFilter) {
-  mComFilterList.Append(aTitle, aFilter);
+  nsString sanitizedFilter(aFilter);
+  sanitizedFilter.ReplaceChar('%', '_');
+
+  if (sanitizedFilter == u"..apps"_ns) {
+    sanitizedFilter = u"*.exe;*.com"_ns;
+  } else {
+    sanitizedFilter.StripWhitespace();
+    if (sanitizedFilter == u"*"_ns) {
+      sanitizedFilter = u"*.*"_ns;
+    }
+  }
+  mFilterList.AppendElement(
+      Filter{.title = nsString(aTitle), .filter = std::move(sanitizedFilter)});
   return NS_OK;
 }
 
@@ -556,14 +500,12 @@ bool nsFilePicker::IsDefaultPathLink() {
   NS_ConvertUTF16toUTF8 ext(mDefaultFilePath);
   ext.Trim(" .", false, true);  // watch out for trailing space and dots
   ToLowerCase(ext);
-  if (StringEndsWith(ext, ".lnk"_ns) || StringEndsWith(ext, ".pif"_ns) ||
-      StringEndsWith(ext, ".url"_ns))
-    return true;
-  return false;
+  return StringEndsWith(ext, ".lnk"_ns) || StringEndsWith(ext, ".pif"_ns) ||
+         StringEndsWith(ext, ".url"_ns);
 }
 
 bool nsFilePicker::IsDefaultPathHtml() {
-  int32_t extIndex = mDefaultFilePath.RFind(".");
+  int32_t extIndex = mDefaultFilePath.RFind(u".");
   if (extIndex >= 0) {
     nsAutoString ext;
     mDefaultFilePath.Right(ext, mDefaultFilePath.Length() - extIndex);
@@ -574,31 +516,3 @@ bool nsFilePicker::IsDefaultPathHtml() {
   }
   return false;
 }
-
-void nsFilePicker::ComDlgFilterSpec::Append(const nsAString& aTitle,
-                                            const nsAString& aFilter) {
-  COMDLG_FILTERSPEC* pSpecForward = mSpecList.AppendElement();
-  if (!pSpecForward) {
-    NS_WARNING("mSpecList realloc failed.");
-    return;
-  }
-  memset(pSpecForward, 0, sizeof(*pSpecForward));
-  nsString* pStr = mStrings.AppendElement(aTitle);
-  if (!pStr) {
-    NS_WARNING("mStrings.AppendElement failed.");
-    return;
-  }
-  pSpecForward->pszName = pStr->get();
-  pStr = mStrings.AppendElement(aFilter);
-  if (!pStr) {
-    NS_WARNING("mStrings.AppendElement failed.");
-    return;
-  }
-  if (aFilter.EqualsLiteral("..apps"))
-    pStr->AssignLiteral("*.exe;*.com");
-  else {
-    pStr->StripWhitespace();
-    if (pStr->EqualsLiteral("*")) pStr->AppendLiteral(".*");
-  }
-  pSpecForward->pszSpec = pStr->get();
-}
